@@ -5,6 +5,7 @@
 #include <keys_nokia.h>
 #include <libintl.h>
 #include <locale.h>
+#include <string.h>
 #include <hildon/hildon.h>
 #include <hildon/hildon-file-chooser-dialog.h>
 
@@ -14,6 +15,7 @@
 #define RINGING_SECTION "[RINGTONE]" // [sic!]
 
 #define PROFILESX_PLUGIN_SETTINGS_FILE "/.profilesx"
+#define VOLUME_SCALE 0.001
 
 struct _editor_data_t
 {
@@ -39,6 +41,9 @@ struct _editor_data_t
   GtkListStore* touchscreensoundLevel;
   GtkListStore* systemsoundLevel;
 
+  GtkWidget* autoAnswerButton;
+  GtkWidget* turnSpeakerOnButton;
+
   GtkWidget* vibratingCheckbutton;
 
   GtkListStore* ringingTone;
@@ -76,6 +81,13 @@ const gchar* get_profile_display_name(const gchar* profile_name)
   }
 }
 
+static double
+_rescale_volume(gdouble volume)
+{
+  gdouble v = volume / 100.0;
+  return v * v * v;
+}
+
 static void 
 read_predefined_ringtones(editor_data_t* data)
 {
@@ -241,14 +253,13 @@ void set_profile_status_icon_value(editor_data_t* data)
     GtkWidget* image;
     if(gtk_icon_theme_has_icon(gtk_icon_theme_get_default (), 
 			       icon_name))
-      {
-	image = gtk_image_new_from_icon_name(icon_name, 18);
-      }
+    {
+      image = gtk_image_new_from_icon_name(icon_name, 18);
+    }
     else
-      {
-	image = gtk_image_new_from_file(icon_name);
-      }
-    
+    {
+      image = gtk_image_new_from_file(icon_name);
+    }
     gtk_button_set_image(GTK_BUTTON(data->iconButton), image);
   }
 }
@@ -274,7 +285,15 @@ set_profile_values(const gchar* profile_name, editor_data_t* data)
   else
     hildon_check_button_set_active(HILDON_CHECK_BUTTON(data->vibratingCheckbutton), FALSE);
 
-  
+  if(data->profile_data->autoanswer)
+    hildon_check_button_set_active(HILDON_CHECK_BUTTON(data->autoAnswerButton), TRUE);
+  else
+    hildon_check_button_set_active(HILDON_CHECK_BUTTON(data->autoAnswerButton), FALSE);
+  if(data->profile_data->turn_speaker_on)
+    hildon_check_button_set_active(HILDON_CHECK_BUTTON(data->turnSpeakerOnButton), TRUE);
+  else
+    hildon_check_button_set_active(HILDON_CHECK_BUTTON(data->turnSpeakerOnButton), FALSE);
+
   set_profile_status_icon_value(data);
 }
 
@@ -341,6 +360,8 @@ save_profile_values(const gchar* profile_name, editor_data_t* data)
 									    0);
 
   data->profile_data->vibrating_enabled = hildon_check_button_get_active(HILDON_CHECK_BUTTON(data->vibratingCheckbutton));
+  data->profile_data->autoanswer = hildon_check_button_get_active(HILDON_CHECK_BUTTON(data->autoAnswerButton));
+  data->profile_data->turn_speaker_on = hildon_check_button_get_active(HILDON_CHECK_BUTTON(data->turnSpeakerOnButton));
   profilesx_store_profile_data(profile_name, data->profile_data);
 }
 
@@ -394,23 +415,25 @@ play_sound_sample(HildonTouchSelector* selector, gint column, editor_data_t* use
 static const gchar*
 get_tone_selector_title(editor_data_t* data, GtkWidget* button)
 {
-  if (data == NULL || button == data->ringtoneButton)
+  if(data)
   {
-    dgettext("osso-profiles", "profi_ti_ringing_tone");
+    if(button == data->ringtoneButton)
+    {
+      return dgettext("osso-profiles", "profi_ti_ringing_tone");
+    }
+    else if(button == data->smstoneButton)
+    {
+      return dgettext("osso-profiles", "profi_ti_sub_text_message_tone");
+    }
+    else if(button == data->imtoneButton)
+    {
+      return dgettext("osso-profiles", "profi_ti_instant_messaging_tone");
+    }
+    else if(button == data->emailtoneButton)
+    {
+      return dgettext("osso-profiles", "profi_ti_email_alert_tone");
+    }  
   }
-  else if (button == data->smstoneButton)
-  {
-    return dgettext("osso-profiles", "profi_ti_sub_text_message_tone");
-  }
-  else if (button == data->imtoneButton)
-  {
-    return dgettext("osso-profiles", "profi_ti_instant_messaging_tone");
-  }
-  else if (button == data->emailtoneButton)
-  {
-    return dgettext("osso-profiles", "profi_ti_email_alert_tone");
-  }  
-
   g_return_val_if_reached(dgettext("osso-profiles", "profi_ti_ringing_tone"));
 }
 
@@ -484,7 +507,8 @@ static
 void
 show_ringtone_selector(GtkWidget* button, editor_data_t *data)
 {
-  g_object_set(data->sound_player, "volume", gtk_range_get_value(GTK_RANGE(data->ringtoneVolumeSlider))/100.0, NULL);
+  gdouble v = _rescale_volume(gtk_range_get_value(GTK_RANGE(data->ringtoneVolumeSlider)));
+  g_object_set(data->sound_player, "volume", v , NULL);
   data->ringtoneSelector=create_tone_selector_widget(data->ringingTone);
   show_tone_selector(data->ringingTone, data->ringtoneButton, data->ringtoneSelector, data);
 }
@@ -493,7 +517,8 @@ static
 void
 show_smstone_selector(GtkWidget* button, editor_data_t *data)
 {
-  g_object_set(data->sound_player, "volume", gtk_range_get_value(GTK_RANGE(data->smsVolumeSlider))/100.0, NULL);
+  gdouble v = _rescale_volume(gtk_range_get_value(GTK_RANGE(data->smsVolumeSlider)));
+  g_object_set(data->sound_player, "volume", v , NULL);
   data->smstoneSelector=create_tone_selector_widget(data->smsTone);
   show_tone_selector(data->smsTone, data->smstoneButton, data->smstoneSelector, data);
 }
@@ -502,7 +527,8 @@ static
 void
 show_imtone_selector(GtkWidget* button, editor_data_t *data)
 {
-  g_object_set(data->sound_player, "volume", gtk_range_get_value(GTK_RANGE(data->imVolumeSlider))/100.0, NULL);
+  gdouble v = _rescale_volume(gtk_range_get_value(GTK_RANGE(data->imVolumeSlider)));
+  g_object_set(data->sound_player, "volume", v, NULL);
   data->imtoneSelector=create_tone_selector_widget(data->imTone);
   show_tone_selector(data->imTone, data->imtoneButton, data->imtoneSelector, data);
 }
@@ -511,7 +537,8 @@ static
 void
 show_emailtone_selector(GtkWidget* button, editor_data_t *data)
 {
-  g_object_set(data->sound_player, "volume", gtk_range_get_value(GTK_RANGE(data->emailVolumeSlider))/100.0, NULL);
+  gdouble v = _rescale_volume(gtk_range_get_value(GTK_RANGE(data->emailVolumeSlider)));
+  g_object_set(data->sound_player, "volume", v, NULL);
   data->emailtoneSelector=create_tone_selector_widget(data->emailTone);
   show_tone_selector(data->emailTone, data->emailtoneButton, data->emailtoneSelector, data);
 }
@@ -789,15 +816,17 @@ layout_dialog_widgets(const gchar* profile_name, editor_data_t* data, GtkWidget*
   if(g_strcmp0(profile_name, "silent")!=0)
   {
     content_area = gtk_vbox_new(FALSE, 3);  
-    gtk_container_add(GTK_CONTAINER(content_area), iconBox);
-    gtk_container_add(GTK_CONTAINER(content_area), data->vibratingCheckbutton);
-    gtk_container_add(GTK_CONTAINER(content_area), ringtoneBox);
-    gtk_container_add(GTK_CONTAINER(content_area), smsBox);
-    gtk_container_add(GTK_CONTAINER(content_area), imBox);
-    gtk_container_add(GTK_CONTAINER(content_area), emailBox);
-    gtk_container_add(GTK_CONTAINER(content_area), data->systemsoundLevelButton);
-    gtk_container_add(GTK_CONTAINER(content_area), data->keypadsoundLevelButton);
-    gtk_container_add(GTK_CONTAINER(content_area), data->touchscreensoundLevelButton);
+    gtk_box_pack_start(GTK_BOX(content_area), iconBox, TRUE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(content_area), data->vibratingCheckbutton, TRUE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(content_area), ringtoneBox, TRUE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(content_area), smsBox, TRUE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(content_area), imBox, TRUE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(content_area), emailBox, TRUE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(content_area), data->systemsoundLevelButton, TRUE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(content_area), data->keypadsoundLevelButton, TRUE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(content_area), data->touchscreensoundLevelButton, TRUE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(content_area), data->autoAnswerButton, TRUE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(content_area), data->turnSpeakerOnButton, TRUE, FALSE, 0);
     pannable_area = hildon_pannable_area_new();
     hildon_pannable_area_add_with_viewport(HILDON_PANNABLE_AREA(pannable_area), content_area);
     gtk_container_add(GTK_CONTAINER(gtk_dialog_get_content_area(GTK_DIALOG(dialog))),
@@ -827,7 +856,6 @@ init_editor_dialog_widgets(editor_data_t* data)
   data->keypadsoundLevel = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_INT);
   data->touchscreensoundLevel = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_INT);
   data->systemsoundLevel = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_INT);
-
   init_sound_level_list_store(data->keypadsoundLevel);
   init_sound_level_list_store(data->systemsoundLevel);
   init_sound_level_list_store(data->touchscreensoundLevel);
@@ -847,6 +875,14 @@ init_editor_dialog_widgets(editor_data_t* data)
   gtk_button_set_label(GTK_BUTTON(data->vibratingCheckbutton),
 		       dgettext("osso-profiles", "profi_fi_general_vibrate"));
   hildon_check_button_set_active(HILDON_CHECK_BUTTON(data->vibratingCheckbutton), TRUE);
+
+  data->autoAnswerButton = hildon_check_button_new(HILDON_SIZE_FINGER_HEIGHT);
+  gtk_button_set_label(GTK_BUTTON(data->autoAnswerButton), "Autoanswer");
+  hildon_check_button_set_active(HILDON_CHECK_BUTTON(data->autoAnswerButton), FALSE);
+  data->turnSpeakerOnButton = hildon_check_button_new(HILDON_SIZE_FINGER_HEIGHT);
+  gtk_button_set_label(GTK_BUTTON(data->turnSpeakerOnButton), 
+		       dgettext("rtcom-call-ui", "call_fi_call_handling_speaker"));
+  hildon_check_button_set_active(HILDON_CHECK_BUTTON(data->turnSpeakerOnButton), FALSE);
 }
 
 void 
