@@ -247,17 +247,18 @@ is_proximity_covered()
 {
   char state[7];
   FILE* fd = fopen("/sys/devices/platform/gpio-switch/proximity/state", "r");
+  gboolean rc = TRUE;
   if(fd != NULL)
   {
-    fread(state, sizeof(char), 7, fd);
-    state[7] = '\0';
+    fread(&state, sizeof(char), 7, fd);
+    state[6] = '\0';
     if(!g_str_has_prefix(state, "closed"))
     {
-      return FALSE;
+      rc = FALSE;
     }
-    fclose( fd );
   }
-  return TRUE;
+  fclose(fd);
+  return rc;
 }
 
 static gboolean
@@ -278,6 +279,7 @@ answer_call(gpointer user_data)
 				 G_TYPE_INVALID, G_TYPE_INVALID);
       g_object_unref(proxy);
     }
+
   }
   return FALSE;
 }
@@ -617,7 +619,7 @@ profilesx_status_plugin_init(ProfilesxStatusPlugin* plugin)
   plugin->priv->button = hildon_button_new(HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT,
 					   HILDON_BUTTON_ARRANGEMENT_VERTICAL);
 
-  hildon_button_set_title(HILDON_BUTTON(plugin->priv->button), dgettext("osso-profiles", "profi_ti_menu_plugin"));
+  hildon_button_set_title(HILDON_BUTTON(plugin->priv->button), "ProfilesX");
   g_signal_connect(plugin->priv->button, "clicked", G_CALLBACK(show_profile_selection_dlg), plugin);
   g_signal_connect(plugin, "unrealize", G_CALLBACK(show_original_profile_button), NULL); 
   GtkWidget *image;
